@@ -15,6 +15,13 @@ fn main() {
     println!("cargo:rerun-if-changed=no_atomic.rs");
     println!("cargo:rerun-if-changed=version.rs");
 
+    #[cfg(feature = "unsafe-assume-single-core")]
+    println!("cargo:rustc-cfg=portable_atomic_unsafe_assume_single_core");
+    #[cfg(feature = "s-mode")]
+    println!("cargo:rustc-cfg=portable_atomic_s_mode");
+    #[cfg(feature = "disable-fiq")]
+    println!("cargo:rustc-cfg=portable_atomic_disable_fiq");
+
     let target = &*env::var("TARGET").expect("TARGET not set");
     let target_arch = &*env::var("CARGO_CFG_TARGET_ARCH").expect("CARGO_CFG_TARGET_ARCH not set");
     let target_os = &*env::var("CARGO_CFG_TARGET_OS").expect("CARGO_CFG_TARGET_OS not set");
@@ -30,7 +37,6 @@ fn main() {
     let mut target_upper = target.replace(|c: char| c == '-' || c == '.', "_");
     target_upper.make_ascii_uppercase();
     println!("cargo:rerun-if-env-changed=CARGO_TARGET_{}_RUSTFLAGS", target_upper);
-    println!("cargo:rerun-if-env-changed=CARGO_CFG_PORTABLE_ATOMIC_NO_OUTLINE_ATOMICS");
 
     let version = match rustc_version() {
         Some(version) => version,
@@ -47,10 +53,6 @@ fn main() {
     // Note that this is `no_`*, not `has_*`. This allows treating as the latest
     // stable rustc is used when the build script doesn't run. This is useful
     // for non-cargo build systems that don't run the build script.
-    // underscore_const_names stabilized in Rust 1.37 (nightly-2019-06-18): https://github.com/rust-lang/rust/pull/61347
-    if !version.probe(37, 2019, 6, 17) {
-        println!("cargo:rustc-cfg=portable_atomic_no_underscore_consts");
-    }
     // atomic_min_max stabilized in Rust 1.45 (nightly-2020-05-30): https://github.com/rust-lang/rust/pull/72324
     if !version.probe(45, 2020, 5, 29) {
         println!("cargo:rustc-cfg=portable_atomic_no_atomic_min_max");
@@ -67,35 +69,35 @@ fn main() {
     if !version.probe(56, 2021, 8, 1) {
         println!("cargo:rustc-cfg=portable_atomic_no_core_unwind_safe");
     }
-    // asm stabilized in Rust 1.59 (nightly-2021-12-16): https://github.com/rust-lang/rust/pull/91728
-    let no_asm = !version.probe(59, 2021, 12, 15);
-    let mut unstable_asm = false;
-    if no_asm {
-        if version.nightly
-            && version.probe(46, 2020, 6, 20)
-            && (target_arch != "x86_64" || version.llvm >= 10)
-            && is_allowed_feature("asm")
-        {
-            // This feature was added in Rust 1.45 (nightly-2020-05-20), but
-            // concat! in asm! requires Rust 1.46 (nightly-2020-06-21).
-            // x86 intel syntax requires LLVM 10.
-            // The part of this feature we use has not been changed since nightly-2020-06-21
-            // until it was stabilized in nightly-2021-12-16, so it can be safely enabled in
-            // nightly, which is older than nightly-2021-12-16.
-            println!("cargo:rustc-cfg=portable_atomic_unstable_asm");
-            unstable_asm = true;
-        }
-        println!("cargo:rustc-cfg=portable_atomic_no_asm");
-    }
-    // aarch64_target_feature stabilized in Rust 1.61 (nightly-2022-03-16): https://github.com/rust-lang/rust/pull/90621
-    if !version.probe(61, 2022, 3, 15) {
-        println!("cargo:rustc-cfg=portable_atomic_no_aarch64_target_feature");
+    // const_raw_ptr_deref stabilized in Rust 1.58 (nightly-2021-11-15): https://github.com/rust-lang/rust/pull/89551
+    if !version.probe(58, 2021, 11, 14) {
+        println!("cargo:rustc-cfg=portable_atomic_no_const_raw_ptr_deref");
     }
     // https://github.com/rust-lang/rust/pull/98383 merged in Rust 1.64 (nightly-2022-07-19).
     if !version.probe(64, 2022, 7, 18) {
         println!("cargo:rustc-cfg=portable_atomic_no_stronger_failure_ordering");
     }
 
+    // asm stabilized in Rust 1.59 (nightly-2021-12-16): https://github.com/rust-lang/rust/pull/91728
+    let no_asm = !version.probe(59, 2021, 12, 15);
+    if no_asm {
+        if version.nightly
+            && version.probe(46, 2020, 6, 20)
+            && ((target_arch != "x86" && target_arch != "x86_64") || version.llvm >= 10)
+            && is_allowed_feature("asm")
+        {
+            // This feature was added in Rust 1.45 (nightly-2020-05-20), but
+            // concat! in asm! requires Rust 1.46 (nightly-2020-06-21).
+            // x86 intel syntax requires LLVM 10 (since Rust 1.53, the minimum
+            // external LLVM version is 10+: https://github.com/rust-lang/rust/pull/83387).
+            // The part of this feature we use has not been changed since nightly-2020-06-21
+            // until it was stabilized in nightly-2021-12-16, so it can be safely enabled in
+            // nightly, which is older than nightly-2021-12-16.
+            println!("cargo:rustc-cfg=portable_atomic_unstable_asm");
+        }
+        println!("cargo:rustc-cfg=portable_atomic_no_asm");
+    }
+
     // feature(cfg_target_has_atomic) stabilized in Rust 1.60 (nightly-2022-02-11): https://github.com/rust-lang/rust/pull/93824
     if !version.probe(60, 2022, 2, 10) {
         if version.nightly
@@ -107,8 +109,8 @@ fn main() {
             // nightly, which is older than nightly-2022-02-11.
             println!("cargo:rustc-cfg=portable_atomic_unstable_cfg_target_has_atomic");
         } else {
-            let target = &*convert_custom_linux_target(target);
             println!("cargo:rustc-cfg=portable_atomic_no_cfg_target_has_atomic");
+            let target = &*convert_custom_linux_target(target);
             if NO_ATOMIC_CAS.contains(&target) {
                 println!("cargo:rustc-cfg=portable_atomic_no_atomic_cas");
             }
@@ -124,19 +126,22 @@ fn main() {
         println!("cargo:rustc-cfg=portable_atomic_no_atomic_load_store");
     }
 
+    if version.llvm >= 16 {
+        println!("cargo:rustc-cfg=portable_atomic_llvm_16");
+    }
     if version.nightly {
-        println!("cargo:rustc-cfg=portable_atomic_nightly");
-
         // https://github.com/rust-lang/rust/pull/97423 merged in Rust 1.64 (nightly-2022-06-30).
         if version.probe(64, 2022, 6, 29) {
             println!("cargo:rustc-cfg=portable_atomic_new_atomic_intrinsics");
         }
-        // https://github.com/rust-lang/rust/pull/96935 merged in Rust 1.64 (nightly-2022-07-07).
-        if version.probe(64, 2022, 7, 6) {
-            println!("cargo:rustc-cfg=portable_atomic_unstable_strict_provenance_atomic_ptr");
+        // https://github.com/rust-lang/rust/pull/100911 (includes https://github.com/rust-lang/stdarch/pull/1315) merged in Rust 1.65 (nightly-2022-08-26).
+        if target_arch == "x86_64" && !version.probe(65, 2022, 8, 25) {
+            println!(
+                "cargo:rustc-cfg=portable_atomic_no_cmpxchg16b_intrinsic_stronger_failure_ordering"
+            );
         }
         // feature(isa_attribute) stabilized in Rust 1.67 (nightly-2022-11-06): https://github.com/rust-lang/rust/pull/102458
-        if !version.probe(67, 2022, 11, 5) {
+        if target_arch == "arm" && !version.probe(67, 2022, 11, 5) {
             println!("cargo:rustc-cfg=portable_atomic_unstable_isa_attribute");
         }
 
@@ -151,40 +156,83 @@ fn main() {
         }
 
         if version.llvm >= 15 {
-            println!("cargo:rustc-cfg=portable_atomic_llvm15");
+            println!("cargo:rustc-cfg=portable_atomic_llvm_15");
         }
+        // https://github.com/rust-lang/rust/pull/93868 merged in Rust 1.60 (nightly-2022-02-13).
+        // https://github.com/rust-lang/rust/pull/111331 merged in Rust 1.71 (nightly-2023-05-09).
         if !no_asm
-            && (target_arch == "powerpc64" || target_arch == "s390x")
+            && (target_arch == "powerpc64" && version.probe(60, 2022, 2, 12)
+                || target_arch == "s390x" && version.probe(71, 2023, 5, 8))
             && is_allowed_feature("asm_experimental_arch")
         {
-            println!("cargo:rustc-cfg=portable_atomic_asm_experimental_arch");
+            println!("cargo:rustc-cfg=portable_atomic_unstable_asm_experimental_arch");
         }
     }
 
+    let is_apple =
+        target_os == "macos" || target_os == "ios" || target_os == "tvos" || target_os == "watchos";
     match target_arch {
         "x86_64" => {
-            // x86_64 macos always support CMPXCHG16B: https://github.com/rust-lang/rust/blob/1.63.0/compiler/rustc_target/src/spec/x86_64_apple_darwin.rs#L7
-            let has_cmpxchg16b = target_os == "macos";
-            // LLVM recognizes this also as cx16 target feature: https://godbolt.org/z/o4Y8W1hcb
-            // It is unlikely that rustc will support that name, so we will ignore it for now.
-            target_feature_if("cmpxchg16b", has_cmpxchg16b, &version, None, true);
-            if version.nightly
-                && (!no_asm || unstable_asm)
-                && cfg!(feature = "fallback")
-                && env::var_os("CARGO_CFG_PORTABLE_ATOMIC_NO_OUTLINE_ATOMICS").is_none()
-                && is_allowed_feature("cmpxchg16b_target_feature")
-            {
-                println!("cargo:rustc-cfg=portable_atomic_cmpxchg16b_dynamic");
+            // cmpxchg16b_target_feature stabilized in Rust 1.69 (nightly-2023-03-01): https://github.com/rust-lang/rust/pull/106774
+            if !version.probe(69, 2023, 2, 28) {
+                if version.nightly && is_allowed_feature("cmpxchg16b_target_feature") {
+                    // This feature has not been changed since 1.33
+                    // (https://github.com/rust-lang/rust/commit/fbb56bcf44d28e65a9495decf091b6d0386e540c)
+                    // until it was stabilized in nightly-2023-03-01, so it can be safely enabled in
+                    // nightly, which is older than nightly-2023-03-01.
+                    println!("cargo:rustc-cfg=portable_atomic_unstable_cmpxchg16b_target_feature");
+                } else {
+                    println!("cargo:rustc-cfg=portable_atomic_no_cmpxchg16b_target_feature");
+                }
             }
+
+            // x86_64 Apple targets always support CMPXCHG16B:
+            // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_target/src/spec/x86_64_apple_darwin.rs#L8
+            // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_target/src/spec/apple_base.rs#L69-L70
+            // Script to get targets that support cmpxchg16b by default:
+            // $ (for target in $(rustc --print target-list); do [[ "${target}" == "x86_64"* ]] && rustc --print cfg --target "${target}" | grep -q cmpxchg16b && echo "${target}"; done)
+            let has_cmpxchg16b = is_apple;
+            // LLVM recognizes this also as cx16 target feature: https://godbolt.org/z/r8zWGcMhd
+            // However, it is unlikely that rustc will support that name, so we ignore it.
+            // cmpxchg16b_target_feature stabilized in Rust 1.69.
+            target_feature_if("cmpxchg16b", has_cmpxchg16b, &version, Some(69), true);
         }
         "aarch64" => {
-            // aarch64 macos always support FEAT_LSE and FEAT_LSE2 because it is armv8.6: https://github.com/rust-lang/rust/blob/1.63.0/compiler/rustc_target/src/spec/aarch64_apple_darwin.rs#L5
+            // aarch64_target_feature stabilized in Rust 1.61 (nightly-2022-03-16): https://github.com/rust-lang/rust/pull/90621
+            if !version.probe(61, 2022, 3, 15) {
+                if version.nightly && is_allowed_feature("aarch64_target_feature") {
+                    // The part of this feature we use has not been changed since 1.27
+                    // (https://github.com/rust-lang/rust/commit/1217d70465edb2079880347fea4baaac56895f51)
+                    // until it was stabilized in nightly-2022-03-16, so it can be safely enabled in
+                    // nightly, which is older than nightly-2022-03-16.
+                    println!("cargo:rustc-cfg=portable_atomic_unstable_aarch64_target_feature");
+                } else {
+                    // On aarch64, when aarch64_target_feature is not available, outline-atomics is also not available.
+                    println!("cargo:rustc-cfg=portable_atomic_no_outline_atomics");
+                }
+            }
+
+            // aarch64 macOS always supports FEAT_LSE and FEAT_LSE2 because it is armv8.5-a:
+            // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/include/llvm/TargetParser/AArch64TargetParser.h#L494
             let is_macos = target_os == "macos";
-            // aarch64_target_feature stabilized in Rust 1.61.
-            target_feature_if("lse", is_macos, &version, Some(61), true);
-            // As of rustc 1.63, target_feature "lse2" is not available on rustc side:
-            // https://github.com/rust-lang/rust/blob/1.63.0/compiler/rustc_codegen_ssa/src/target_features.rs#L45
+            let mut has_lse = is_macos;
+            // FEAT_LSE2 doesn't imply FEAT_LSE. FEAT_LSE128 implies FEAT_LSE but not FEAT_LSE2.
+            // As of rustc 1.70, target_feature "lse2"/"lse128"/"rcpc3" is not available on rustc side:
+            // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_codegen_ssa/src/target_features.rs#L58
             target_feature_if("lse2", is_macos, &version, None, false);
+            // LLVM supports FEAT_LRCPC3 and FEAT_LSE128 on LLVM 16+:
+            // https://github.com/llvm/llvm-project/commit/a6aaa969f7caec58a994142f8d855861cf3a1463
+            // https://github.com/llvm/llvm-project/commit/7fea6f2e0e606e5339c3359568f680eaf64aa306
+            has_lse |= target_feature_if("lse128", false, &version, None, false);
+            target_feature_if("rcpc3", false, &version, None, false);
+            // aarch64_target_feature stabilized in Rust 1.61.
+            target_feature_if("lse", has_lse, &version, Some(61), true);
+
+            // As of Apple M1/M1 Pro, on Apple hardware, CAS loop-based RMW is much slower than LL/SC
+            // loop-based RMW: https://github.com/taiki-e/portable-atomic/pull/89
+            if is_apple {
+                println!("cargo:rustc-cfg=portable_atomic_ll_sc_rmw");
+            }
         }
         "arm" => {
             // #[cfg(target_feature = "v7")] and others don't work on stable.
@@ -199,12 +247,15 @@ fn main() {
             // See https://github.com/taiki-e/atomic-maybe-uninit/blob/HEAD/build.rs for details
             let mut is_mclass = false;
             match subarch {
-                "v7" | "v7a" | "v7neon" | "v7s" | "v7k" | "v8a" => {} // aclass
+                "v7" | "v7a" | "v7neon" | "v7s" | "v7k" | "v8a" | "v9a" => {} // aclass
                 "v6m" | "v7em" | "v7m" | "v8m" => is_mclass = true,
                 "v7r" | "v8r" => {} // rclass
                 // arm-linux-androideabi is v5te
-                // https://github.com/rust-lang/rust/blob/1.63.0/compiler/rustc_target/src/spec/arm_linux_androideabi.rs#L11-L12
+                // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_target/src/spec/arm_linux_androideabi.rs#L11-L12
                 _ if target == "arm-linux-androideabi" => subarch = "v5te",
+                // armeb-unknown-linux-gnueabi is v8
+                // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_target/src/spec/armeb_unknown_linux_gnueabi.rs#L12
+                _ if target == "armeb-unknown-linux-gnueabi" => subarch = "v8",
                 // v6 targets other than v6m don't have *class target feature.
                 "" | "v6" | "v6k" => subarch = "v6",
                 // Other targets don't have *class target feature.
@@ -222,13 +273,15 @@ fn main() {
             let v6 = known
                 && (subarch.starts_with("v6")
                     || subarch.starts_with("v7")
-                    || subarch.starts_with("v8"));
+                    || subarch.starts_with("v8")
+                    || subarch.starts_with("v9"));
             target_feature_if("v6", v6, &version, None, true);
         }
         "powerpc64" => {
             let target_endian =
                 env::var("CARGO_CFG_TARGET_ENDIAN").expect("CARGO_CFG_TARGET_ENDIAN not set");
-            // powerpc64le is pwr8+ by default https://github.com/llvm/llvm-project/blob/llvmorg-15.0.0/llvm/lib/Target/PowerPC/PPC.td#L652
+            // powerpc64le is pwr8+ by default
+            // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/PowerPC/PPC.td#L663
             // See also https://github.com/rust-lang/rust/issues/59932
             let mut has_pwr8_features = target_endian == "little";
             // https://github.com/llvm/llvm-project/commit/549e118e93c666914a1045fde38a2cac33e1e445
@@ -239,14 +292,50 @@ fn main() {
                         has_pwr8_features = cpu_version >= 8;
                     }
                 } else {
-                    // https://github.com/llvm/llvm-project/blob/llvmorg-15.0.0/llvm/lib/Target/PowerPC/PPC.td#L652
-                    // https://github.com/llvm/llvm-project/blob/llvmorg-15.0.0/llvm/lib/Target/PowerPC/PPC.td#L434-L436
+                    // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/PowerPC/PPC.td#L663
+                    // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/PowerPC/PPC.td#L445-L447
                     has_pwr8_features = cpu == "ppc64le" || cpu == "future";
                 }
             }
+            // Note: As of rustc 1.70, target_feature "quadword-atomics" is not available on rustc side:
+            // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_codegen_ssa/src/target_features.rs#L226
             // lqarx and stqcx.
             target_feature_if("quadword-atomics", has_pwr8_features, &version, None, false);
         }
+        "s390x" => {
+            // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/SystemZ/SystemZFeatures.td
+            let mut has_arch9_features = false; // z196+
+            let mut has_arch13_features = false; // z15+
+            if let Some(cpu) = target_cpu() {
+                // https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/SystemZ/SystemZProcessors.td
+                match &*cpu {
+                    "arch9" | "z196" | "arch10" | "zEC12" | "arch11" | "z13" | "arch12" | "z14" => {
+                        has_arch9_features = true;
+                    }
+                    "arch13" | "z15" | "arch14" | "z16" => {
+                        has_arch9_features = true;
+                        has_arch13_features = true;
+                    }
+                    _ => {}
+                }
+            }
+            // Note: As of rustc 1.70, target_feature "fast-serialization"/"load-store-on-cond"/"distinct-ops"/"miscellaneous-extensions-3" is not available on rustc side:
+            // https://github.com/rust-lang/rust/blob/1.70.0/compiler/rustc_codegen_ssa/src/target_features.rs
+            // bcr 14,0
+            target_feature_if("fast-serialization", has_arch9_features, &version, None, false);
+            // {l,st}oc{,g}{,r}
+            target_feature_if("load-store-on-cond", has_arch9_features, &version, None, false);
+            // {al,sl,n,o,x}{,g}rk
+            target_feature_if("distinct-ops", has_arch9_features, &version, None, false);
+            // nand (nnr{,g}k), select (sel{,g}r), etc.
+            target_feature_if(
+                "miscellaneous-extensions-3",
+                has_arch13_features,
+                &version,
+                None,
+                false,
+            );
+        }
         _ => {}
     }
 }
@@ -257,7 +346,7 @@ fn target_feature_if(
     version: &Version,
     stabilized: Option<u32>,
     is_rustc_target_feature: bool,
-) {
+) -> bool {
     // HACK: Currently, it seems that the only way to handle unstable target
     // features on the stable is to parse the `-C target-feature` in RUSTFLAGS.
     //
@@ -265,14 +354,14 @@ fn target_feature_if(
     // - CARGO_CFG_TARGET_FEATURE excludes unstable target features on stable.
     //
     // As mentioned in the [RFC2045], unstable target features are also passed to LLVM
-    // (e.g., https://godbolt.org/z/8Eh3z5Wzb), so this hack works properly on stable.
+    // (e.g., https://godbolt.org/z/TfaEx95jc), so this hack works properly on stable.
     //
     // [RFC2045]: https://rust-lang.github.io/rfcs/2045-target-feature.html#backend-compilation-options
     if is_rustc_target_feature
         && (version.nightly || stabilized.map_or(false, |stabilized| version.minor >= stabilized))
     {
         // In this case, cfg(target_feature = "...") would work, so skip emitting our own target_feature cfg.
-        return;
+        return false;
     } else if let Some(rustflags) = env::var_os("CARGO_ENCODED_RUSTFLAGS") {
         for mut flag in rustflags.to_string_lossy().split('\x1f') {
             flag = strip_prefix(flag, "-C").unwrap_or(flag);
@@ -292,6 +381,7 @@ fn target_feature_if(
     if has_target_feature {
         println!("cargo:rustc-cfg=portable_atomic_target_feature=\"{}\"", name);
     }
+    has_target_feature
 }
 
 fn target_cpu() -> Option<String> {
