@@ -1,20 +1,19 @@
 // Adapted from https://github.com/rust-embedded/cortex-m.
 //
 // Generated asm:
-// - armv6-m https://godbolt.org/z/oMjd4h3Tr
+// - armv6-m https://godbolt.org/z/sTezYnaj9
 
 #[cfg(not(portable_atomic_no_asm))]
 use core::arch::asm;
 
 pub(super) use core::sync::atomic;
 
-#[derive(Clone, Copy)]
-pub(super) struct State(u32);
+pub(super) type State = u32;
 
 /// Disables interrupts and returns the previous interrupt state.
 #[inline]
 pub(super) fn disable() -> State {
-    let r: u32;
+    let r: State;
     // SAFETY: reading the priority mask register and disabling interrupts are safe.
     // (see module-level comments of interrupt/mod.rs on the safety of using privileged instructions)
     unsafe {
@@ -26,12 +25,16 @@ pub(super) fn disable() -> State {
             options(nostack, preserves_flags),
         );
     }
-    State(r)
+    r
 }
 
 /// Restores the previous interrupt state.
+///
+/// # Safety
+///
+/// The state must be the one retrieved by the previous `disable`.
 #[inline]
-pub(super) unsafe fn restore(State(r): State) {
+pub(super) unsafe fn restore(r: State) {
     if r & 0x1 == 0 {
         // SAFETY: the caller must guarantee that the state was retrieved by the previous `disable`,
         // and we've checked that interrupts were enabled before disabling interrupts.
