@@ -1,6 +1,13 @@
 // Critical section based fallback implementations
 //
-// Critical session (disabling interrupts) based fallback is not sound on multi-core systems.
+// This module supports two different critical section implementations:
+// - Built-in "disable all interrupts".
+// - Call into the `critical-section` crate (which allows the user to plug any implementation).
+//
+// The `critical-section`-based fallback is enabled when the user asks for it with the `critical-section`
+// Cargo feature.
+//
+// The "disable interrupts" fallback is not sound on multi-core systems.
 // Also, this uses privileged instructions to disable interrupts, so it usually
 // doesn't work on unprivileged mode. Using this fallback in an environment where privileged
 // instructions are not available is also usually considered **unsound**,
@@ -17,10 +24,10 @@
 // interrupts [^avr2] in atomic ops by default, is considered the latter.
 // MSP430 as well.
 //
-// See also README.md of this module.
+// See also README.md of this directory.
 //
-// [^avr1]: https://github.com/llvm/llvm-project/blob/llvmorg-15.0.0/llvm/lib/Target/AVR/AVRExpandPseudoInsts.cpp#L1008
-// [^avr2]: https://github.com/llvm/llvm-project/blob/llvmorg-15.0.0/llvm/test/CodeGen/AVR/atomics/load16.ll#L5
+// [^avr1]: https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/lib/Target/AVR/AVRExpandPseudoInsts.cpp#L1074
+// [^avr2]: https://github.com/llvm/llvm-project/blob/llvmorg-17.0.0-rc2/llvm/test/CodeGen/AVR/atomics/load16.ll#L5
 
 // On some platforms, atomic load/store can be implemented in a more efficient
 // way than disabling interrupts. On MSP430, some RMWs that do not return the
@@ -30,35 +37,52 @@
 // CAS together with atomic load/store. The load/store will not be
 // called while interrupts are disabled, and since the load/store is
 // atomic, it is not affected by interrupts even if interrupts are enabled.
-#[cfg(not(target_arch = "avr"))]
+#[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
 use arch::atomic;
 
+#[cfg(not(feature = "critical-section"))]
 #[cfg_attr(
     all(
         target_arch = "arm",
-        any(target_feature = "mclass", portable_atomic_target_feature = "mclass")
+        any(target_feature = "mclass", portable_atomic_target_feature = "mclass"),
     ),
     path = "armv6m.rs"
 )]
 #[cfg_attr(
     all(
         target_arch = "arm",
-        not(any(target_feature = "mclass", portable_atomic_target_feature = "mclass"))
+        not(any(target_feature = "mclass", portable_atomic_target_feature = "mclass")),
     ),
     path = "armv4t.rs"
 )]
 #[cfg_attr(target_arch = "avr", path = "avr.rs")]
 #[cfg_attr(target_arch = "msp430", path = "msp430.rs")]
 #[cfg_attr(any(target_arch = "riscv32", target_arch = "riscv64"), path = "riscv.rs")]
+#[cfg_attr(target_arch = "xtensa", path = "xtensa.rs")]
 mod arch;
 
 use core::{cell::UnsafeCell, sync::atomic::Ordering};
 
+// Critical section implementations might use locks internally.
+#[cfg(feature = "critical-section")]
+const IS_ALWAYS_LOCK_FREE: bool = false;
+
 // Consider atomic operations based on disabling interrupts on single-core
 // systems are lock-free. (We consider the pre-v6 ARM Linux's atomic operations
 // provided in a similar way by the Linux kernel to be lock-free.)
+#[cfg(not(feature = "critical-section"))]
 const IS_ALWAYS_LOCK_FREE: bool = true;
 
+#[cfg(feature = "critical-section")]
+#[inline]
+fn with<F, R>(f: F) -> R
+where
+    F: FnOnce() -> R,
+{
+    critical_section::with(|_| f())
+}
+
+#[cfg(not(feature = "critical-section"))]
 #[inline]
 fn with<F, R>(f: F) -> R
 where
@@ -76,204 +100,6 @@ where
     r
 }
 
-#[repr(C, align(1))]
-pub(crate) struct AtomicBool {
-    v: UnsafeCell<u8>,
-}
-
-// Send is implicitly implemented.
-// SAFETY: any data races are prevented by disabling interrupts or
-// atomic intrinsics (see module-level comments) and the raw pointer is
-// valid because we got it from a reference.
-unsafe impl Sync for AtomicBool {}
-
-impl AtomicBool {
-    #[inline]
-    pub(crate) const fn new(v: bool) -> Self {
-        Self { v: UnsafeCell::new(v as u8) }
-    }
-
-    #[inline]
-    pub(crate) fn is_lock_free() -> bool {
-        Self::is_always_lock_free()
-    }
-    #[inline]
-    pub(crate) const fn is_always_lock_free() -> bool {
-        IS_ALWAYS_LOCK_FREE
-    }
-
-    #[inline]
-    pub(crate) fn get_mut(&mut self) -> &mut bool {
-        // SAFETY: the mutable reference guarantees unique ownership.
-        unsafe { &mut *(self.v.get() as *mut bool) }
-    }
-
-    #[inline]
-    pub(crate) fn into_inner(self) -> bool {
-        self.v.into_inner() != 0
-    }
-
-    #[inline]
-    #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
-    pub(crate) fn load(&self, order: Ordering) -> bool {
-        crate::utils::assert_load_ordering(order);
-        #[deny(unreachable_patterns)]
-        match () {
-            #[cfg(not(target_arch = "avr"))]
-            // SAFETY: any data races are prevented by atomic intrinsics (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => unsafe { (*(self as *const Self as *const atomic::AtomicBool)).load(order) },
-            #[cfg(target_arch = "avr")]
-            // SAFETY: any data races are prevented by disabling interrupts (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => with(|| unsafe { self.v.get().read() != 0 }),
-        }
-    }
-
-    #[inline]
-    #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
-    pub(crate) fn store(&self, val: bool, order: Ordering) {
-        crate::utils::assert_store_ordering(order);
-        #[deny(unreachable_patterns)]
-        match () {
-            #[cfg(not(target_arch = "avr"))]
-            // SAFETY: any data races are prevented by atomic intrinsics (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => unsafe {
-                (*(self as *const Self as *const atomic::AtomicBool)).store(val, order);
-            },
-            #[cfg(target_arch = "avr")]
-            // SAFETY: any data races are prevented by disabling interrupts (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => with(|| unsafe { self.v.get().write(val as u8) }),
-        }
-    }
-
-    #[inline]
-    pub(crate) fn swap(&self, val: bool, _order: Ordering) -> bool {
-        // SAFETY: any data races are prevented by disabling interrupts (see
-        // module-level comments) and the raw pointer is valid because we got it
-        // from a reference.
-        with(|| unsafe { self.v.get().replace(val as u8) != 0 })
-    }
-
-    #[inline]
-    #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
-    pub(crate) fn compare_exchange(
-        &self,
-        current: bool,
-        new: bool,
-        success: Ordering,
-        failure: Ordering,
-    ) -> Result<bool, bool> {
-        crate::utils::assert_compare_exchange_ordering(success, failure);
-        // SAFETY: any data races are prevented by disabling interrupts (see
-        // module-level comments) and the raw pointer is valid because we got it
-        // from a reference.
-        with(|| unsafe {
-            let result = self.v.get().read();
-            if result == current as u8 {
-                self.v.get().write(new as u8);
-                Ok(result != 0)
-            } else {
-                Err(result != 0)
-            }
-        })
-    }
-
-    #[inline]
-    #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
-    pub(crate) fn compare_exchange_weak(
-        &self,
-        current: bool,
-        new: bool,
-        success: Ordering,
-        failure: Ordering,
-    ) -> Result<bool, bool> {
-        self.compare_exchange(current, new, success, failure)
-    }
-
-    #[inline]
-    pub(crate) fn fetch_and(&self, val: bool, _order: Ordering) -> bool {
-        // SAFETY: any data races are prevented by disabling interrupts (see
-        // module-level comments) and the raw pointer is valid because we got it
-        // from a reference.
-        with(|| unsafe {
-            let result = self.v.get().read();
-            self.v.get().write(result & val as u8);
-            result != 0
-        })
-    }
-
-    #[inline]
-    pub(crate) fn fetch_nand(&self, val: bool, order: Ordering) -> bool {
-        if val {
-            // !(x & true) == !x
-            // We must invert the bool.
-            self.fetch_xor(true, order)
-        } else {
-            // !(x & false) == true
-            // We must set the bool to true.
-            self.swap(true, order)
-        }
-    }
-
-    #[inline]
-    pub(crate) fn fetch_or(&self, val: bool, _order: Ordering) -> bool {
-        // SAFETY: any data races are prevented by disabling interrupts (see
-        // module-level comments) and the raw pointer is valid because we got it
-        // from a reference.
-        with(|| unsafe {
-            let result = self.v.get().read();
-            self.v.get().write(result | val as u8);
-            result != 0
-        })
-    }
-
-    #[inline]
-    pub(crate) fn fetch_xor(&self, val: bool, _order: Ordering) -> bool {
-        // SAFETY: any data races are prevented by disabling interrupts (see
-        // module-level comments) and the raw pointer is valid because we got it
-        // from a reference.
-        with(|| unsafe {
-            let result = self.v.get().read();
-            self.v.get().write(result ^ val as u8);
-            result != 0
-        })
-    }
-}
-
-#[cfg(not(target_arch = "msp430"))]
-no_fetch_ops_impl!(AtomicBool, bool);
-#[cfg(target_arch = "msp430")]
-impl AtomicBool {
-    #[inline]
-    pub(crate) fn and(&self, val: bool, order: Ordering) {
-        // SAFETY: Self and atomic::AtomicBool have the same layout,
-        unsafe {
-            (*(self as *const Self as *const atomic::AtomicBool)).and(val, order);
-        }
-    }
-    #[inline]
-    pub(crate) fn or(&self, val: bool, order: Ordering) {
-        // SAFETY: Self and atomic::AtomicBool have the same layout,
-        unsafe {
-            (*(self as *const Self as *const atomic::AtomicBool)).or(val, order);
-        }
-    }
-    #[inline]
-    pub(crate) fn xor(&self, val: bool, order: Ordering) {
-        // SAFETY: Self and atomic::AtomicBool have the same layout,
-        unsafe {
-            (*(self as *const Self as *const atomic::AtomicBool)).xor(val, order);
-        }
-    }
-}
-
 #[cfg_attr(target_pointer_width = "16", repr(C, align(2)))]
 #[cfg_attr(target_pointer_width = "32", repr(C, align(4)))]
 #[cfg_attr(target_pointer_width = "64", repr(C, align(8)))]
@@ -283,12 +109,10 @@ pub(crate) struct AtomicPtr<T> {
 }
 
 // SAFETY: any data races are prevented by disabling interrupts or
-// atomic intrinsics (see module-level comments) and the raw pointer is
-// valid because we got it from a reference.
+// atomic intrinsics (see module-level comments).
 unsafe impl<T> Send for AtomicPtr<T> {}
 // SAFETY: any data races are prevented by disabling interrupts or
-// atomic intrinsics (see module-level comments) and the raw pointer is
-// valid because we got it from a reference.
+// atomic intrinsics (see module-level comments).
 unsafe impl<T> Sync for AtomicPtr<T> {}
 
 impl<T> AtomicPtr<T> {
@@ -322,40 +146,30 @@ impl<T> AtomicPtr<T> {
     #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
     pub(crate) fn load(&self, order: Ordering) -> *mut T {
         crate::utils::assert_load_ordering(order);
-        #[deny(unreachable_patterns)]
-        match () {
-            #[cfg(not(target_arch = "avr"))]
-            // SAFETY: any data races are prevented by atomic intrinsics (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => unsafe { (*(self as *const Self as *const atomic::AtomicPtr<T>)).load(order) },
-            #[cfg(target_arch = "avr")]
-            // SAFETY: any data races are prevented by disabling interrupts (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => with(|| unsafe { self.p.get().read() }),
+        #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+        {
+            self.as_native().load(order)
         }
+        #[cfg(any(target_arch = "avr", feature = "critical-section"))]
+        // SAFETY: any data races are prevented by disabling interrupts (see
+        // module-level comments) and the raw pointer is valid because we got it
+        // from a reference.
+        with(|| unsafe { self.p.get().read() })
     }
 
     #[inline]
     #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
     pub(crate) fn store(&self, ptr: *mut T, order: Ordering) {
         crate::utils::assert_store_ordering(order);
-        #[deny(unreachable_patterns)]
-        match () {
-            #[cfg(not(target_arch = "avr"))]
-            // SAFETY: any data races are prevented by atomic intrinsics (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => unsafe {
-                (*(self as *const Self as *const atomic::AtomicPtr<T>)).store(ptr, order);
-            },
-            #[cfg(target_arch = "avr")]
-            // SAFETY: any data races are prevented by disabling interrupts (see
-            // module-level comments) and the raw pointer is valid because we got it
-            // from a reference.
-            () => with(|| unsafe { self.p.get().write(ptr) }),
+        #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+        {
+            self.as_native().store(ptr, order);
         }
+        #[cfg(any(target_arch = "avr", feature = "critical-section"))]
+        // SAFETY: any data races are prevented by disabling interrupts (see
+        // module-level comments) and the raw pointer is valid because we got it
+        // from a reference.
+        with(|| unsafe { self.p.get().write(ptr) });
     }
 
     #[inline]
@@ -380,12 +194,12 @@ impl<T> AtomicPtr<T> {
         // module-level comments) and the raw pointer is valid because we got it
         // from a reference.
         with(|| unsafe {
-            let result = self.p.get().read();
-            if result == current {
+            let prev = self.p.get().read();
+            if prev == current {
                 self.p.get().write(new);
-                Ok(result)
+                Ok(prev)
             } else {
-                Err(result)
+                Err(prev)
             }
         })
     }
@@ -401,10 +215,23 @@ impl<T> AtomicPtr<T> {
     ) -> Result<*mut T, *mut T> {
         self.compare_exchange(current, new, success, failure)
     }
+
+    #[inline]
+    pub(crate) const fn as_ptr(&self) -> *mut *mut T {
+        self.p.get()
+    }
+
+    #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+    #[inline]
+    fn as_native(&self) -> &atomic::AtomicPtr<T> {
+        // SAFETY: AtomicPtr and atomic::AtomicPtr have the same layout and
+        // guarantee atomicity in a compatible way. (see module-level comments)
+        unsafe { &*(self as *const Self as *const atomic::AtomicPtr<T>) }
+    }
 }
 
 macro_rules! atomic_int {
-    (base, $atomic_type:ident, $int_type:ident, $align:expr) => {
+    (base, $atomic_type:ident, $int_type:ident, $align:literal) => {
         #[repr(C, align($align))]
         pub(crate) struct $atomic_type {
             v: UnsafeCell<$int_type>,
@@ -412,8 +239,7 @@ macro_rules! atomic_int {
 
         // Send is implicitly implemented.
         // SAFETY: any data races are prevented by disabling interrupts or
-        // atomic intrinsics (see module-level comments) and the raw pointer is
-        // valid because we got it from a reference.
+        // atomic intrinsics (see module-level comments).
         unsafe impl Sync for $atomic_type {}
 
         impl $atomic_type {
@@ -442,116 +268,99 @@ macro_rules! atomic_int {
             pub(crate) fn into_inner(self) -> $int_type {
                 self.v.into_inner()
             }
+
+            #[inline]
+            pub(crate) const fn as_ptr(&self) -> *mut $int_type {
+                self.v.get()
+            }
         }
     };
-    ($kind:ident, load_store_atomic, $atomic_type:ident, $int_type:ident, $align:expr) => {
+    (load_store_atomic, $atomic_type:ident, $int_type:ident, $align:literal) => {
         atomic_int!(base, $atomic_type, $int_type, $align);
-        atomic_int!($kind, cas, $atomic_type, $int_type);
+        atomic_int!(cas, $atomic_type, $int_type);
         impl $atomic_type {
             #[inline]
             #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
             pub(crate) fn load(&self, order: Ordering) -> $int_type {
                 crate::utils::assert_load_ordering(order);
-                #[deny(unreachable_patterns)]
-                match () {
-                    #[cfg(not(target_arch = "avr"))]
-                    // SAFETY: any data races are prevented by atomic intrinsics (see
-                    // module-level comments) and the raw pointer is valid because we got it
-                    // from a reference.
-                    () => unsafe {
-                        (*(self as *const Self as *const atomic::$atomic_type)).load(order)
-                    },
-                    #[cfg(target_arch = "avr")]
-                    // SAFETY: any data races are prevented by disabling interrupts (see
-                    // module-level comments) and the raw pointer is valid because we got it
-                    // from a reference.
-                    () => with(|| unsafe { self.v.get().read() }),
+                #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+                {
+                    self.as_native().load(order)
                 }
+                #[cfg(any(target_arch = "avr", feature = "critical-section"))]
+                // SAFETY: any data races are prevented by disabling interrupts (see
+                // module-level comments) and the raw pointer is valid because we got it
+                // from a reference.
+                with(|| unsafe { self.v.get().read() })
             }
 
             #[inline]
             #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
             pub(crate) fn store(&self, val: $int_type, order: Ordering) {
                 crate::utils::assert_store_ordering(order);
-                #[deny(unreachable_patterns)]
-                match () {
-                    #[cfg(not(target_arch = "avr"))]
-                    // SAFETY: any data races are prevented by atomic intrinsics (see
-                    // module-level comments) and the raw pointer is valid because we got it
-                    // from a reference.
-                    () => unsafe {
-                        (*(self as *const Self as *const atomic::$atomic_type)).store(val, order);
-                    },
-                    #[cfg(target_arch = "avr")]
-                    // SAFETY: any data races are prevented by disabling interrupts (see
-                    // module-level comments) and the raw pointer is valid because we got it
-                    // from a reference.
-                    () => with(|| unsafe { self.v.get().write(val) }),
+                #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+                {
+                    self.as_native().store(val, order);
                 }
+                #[cfg(any(target_arch = "avr", feature = "critical-section"))]
+                // SAFETY: any data races are prevented by disabling interrupts (see
+                // module-level comments) and the raw pointer is valid because we got it
+                // from a reference.
+                with(|| unsafe { self.v.get().write(val) });
+            }
+
+            #[cfg(not(any(target_arch = "avr", feature = "critical-section")))]
+            #[inline]
+            fn as_native(&self) -> &atomic::$atomic_type {
+                // SAFETY: $atomic_type and atomic::$atomic_type have the same layout and
+                // guarantee atomicity in a compatible way. (see module-level comments)
+                unsafe { &*(self as *const Self as *const atomic::$atomic_type) }
             }
         }
 
-        #[cfg(not(target_arch = "msp430"))]
-        no_fetch_ops_impl!($atomic_type, $int_type);
-        #[cfg(not(target_arch = "msp430"))]
+        #[cfg(not(all(target_arch = "msp430", not(feature = "critical-section"))))]
+        impl_default_no_fetch_ops!($atomic_type, $int_type);
+        impl_default_bit_opts!($atomic_type, $int_type);
+        #[cfg(not(all(target_arch = "msp430", not(feature = "critical-section"))))]
         impl $atomic_type {
             #[inline]
             pub(crate) fn not(&self, order: Ordering) {
                 self.fetch_not(order);
             }
         }
-        #[cfg(target_arch = "msp430")]
+        #[cfg(all(target_arch = "msp430", not(feature = "critical-section")))]
         impl $atomic_type {
             #[inline]
             pub(crate) fn add(&self, val: $int_type, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).add(val, order);
-                }
+                self.as_native().add(val, order);
             }
             #[inline]
             pub(crate) fn sub(&self, val: $int_type, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).sub(val, order);
-                }
+                self.as_native().sub(val, order);
             }
             #[inline]
             pub(crate) fn and(&self, val: $int_type, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).and(val, order);
-                }
+                self.as_native().and(val, order);
             }
             #[inline]
             pub(crate) fn or(&self, val: $int_type, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).or(val, order);
-                }
+                self.as_native().or(val, order);
             }
             #[inline]
             pub(crate) fn xor(&self, val: $int_type, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).xor(val, order);
-                }
+                self.as_native().xor(val, order);
             }
             #[inline]
             pub(crate) fn not(&self, order: Ordering) {
-                // SAFETY: Self and atomic::$atomic_type have the same layout,
-                unsafe {
-                    (*(self as *const Self as *const atomic::$atomic_type)).not(order);
-                }
+                self.as_native().not(order);
             }
         }
     };
-    (
-        $kind:ident, load_store_critical_session, $atomic_type:ident, $int_type:ident, $align:expr
-    ) => {
+    (load_store_critical_session, $atomic_type:ident, $int_type:ident, $align:literal) => {
         atomic_int!(base, $atomic_type, $int_type, $align);
-        atomic_int!($kind, cas, $atomic_type, $int_type);
-        no_fetch_ops_impl!($atomic_type, $int_type);
+        atomic_int!(cas, $atomic_type, $int_type);
+        impl_default_no_fetch_ops!($atomic_type, $int_type);
+        impl_default_bit_opts!($atomic_type, $int_type);
         impl $atomic_type {
             #[inline]
             #[cfg_attr(all(debug_assertions, not(portable_atomic_no_track_caller)), track_caller)]
@@ -579,7 +388,7 @@ macro_rules! atomic_int {
             }
         }
     };
-    (uint, cas, $atomic_type:ident, $int_type:ident) => {
+    (cas, $atomic_type:ident, $int_type:ident) => {
         impl $atomic_type {
             #[inline]
             pub(crate) fn swap(&self, val: $int_type, _order: Ordering) -> $int_type {
@@ -603,12 +412,12 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    if result == current {
+                    let prev = self.v.get().read();
+                    if prev == current {
                         self.v.get().write(new);
-                        Ok(result)
+                        Ok(prev)
                     } else {
-                        Err(result)
+                        Err(prev)
                     }
                 })
             }
@@ -631,9 +440,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result.wrapping_add(val));
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev.wrapping_add(val));
+                    prev
                 })
             }
 
@@ -643,9 +452,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result.wrapping_sub(val));
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev.wrapping_sub(val));
+                    prev
                 })
             }
 
@@ -655,9 +464,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result & val);
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev & val);
+                    prev
                 })
             }
 
@@ -667,9 +476,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(!(result & val));
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(!(prev & val));
+                    prev
                 })
             }
 
@@ -679,9 +488,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result | val);
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev | val);
+                    prev
                 })
             }
 
@@ -691,9 +500,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result ^ val);
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev ^ val);
+                    prev
                 })
             }
 
@@ -703,9 +512,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(core::cmp::max(result, val));
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(core::cmp::max(prev, val));
+                    prev
                 })
             }
 
@@ -715,9 +524,9 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(core::cmp::min(result, val));
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(core::cmp::min(prev, val));
+                    prev
                 })
             }
 
@@ -727,25 +536,21 @@ macro_rules! atomic_int {
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(!result);
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(!prev);
+                    prev
                 })
             }
-        }
-    };
-    (int, cas, $atomic_type:ident, $int_type:ident) => {
-        atomic_int!(uint, cas, $atomic_type, $int_type);
-        impl $atomic_type {
+
             #[inline]
             pub(crate) fn fetch_neg(&self, _order: Ordering) -> $int_type {
                 // SAFETY: any data races are prevented by disabling interrupts (see
                 // module-level comments) and the raw pointer is valid because we got it
                 // from a reference.
                 with(|| unsafe {
-                    let result = self.v.get().read();
-                    self.v.get().write(result.wrapping_neg());
-                    result
+                    let prev = self.v.get().read();
+                    self.v.get().write(prev.wrapping_neg());
+                    prev
                 })
             }
             #[inline]
@@ -757,59 +562,58 @@ macro_rules! atomic_int {
 }
 
 #[cfg(target_pointer_width = "16")]
-atomic_int!(int, load_store_atomic, AtomicIsize, isize, 2);
+atomic_int!(load_store_atomic, AtomicIsize, isize, 2);
 #[cfg(target_pointer_width = "16")]
-atomic_int!(uint, load_store_atomic, AtomicUsize, usize, 2);
+atomic_int!(load_store_atomic, AtomicUsize, usize, 2);
 #[cfg(target_pointer_width = "32")]
-atomic_int!(int, load_store_atomic, AtomicIsize, isize, 4);
+atomic_int!(load_store_atomic, AtomicIsize, isize, 4);
 #[cfg(target_pointer_width = "32")]
-atomic_int!(uint, load_store_atomic, AtomicUsize, usize, 4);
+atomic_int!(load_store_atomic, AtomicUsize, usize, 4);
 #[cfg(target_pointer_width = "64")]
-atomic_int!(int, load_store_atomic, AtomicIsize, isize, 8);
+atomic_int!(load_store_atomic, AtomicIsize, isize, 8);
 #[cfg(target_pointer_width = "64")]
-atomic_int!(uint, load_store_atomic, AtomicUsize, usize, 8);
+atomic_int!(load_store_atomic, AtomicUsize, usize, 8);
 #[cfg(target_pointer_width = "128")]
-atomic_int!(int, load_store_atomic, AtomicIsize, isize, 16);
+atomic_int!(load_store_atomic, AtomicIsize, isize, 16);
 #[cfg(target_pointer_width = "128")]
-atomic_int!(uint, load_store_atomic, AtomicUsize, usize, 16);
+atomic_int!(load_store_atomic, AtomicUsize, usize, 16);
 
-atomic_int!(int, load_store_atomic, AtomicI8, i8, 1);
-atomic_int!(uint, load_store_atomic, AtomicU8, u8, 1);
-atomic_int!(int, load_store_atomic, AtomicI16, i16, 2);
-atomic_int!(uint, load_store_atomic, AtomicU16, u16, 2);
+atomic_int!(load_store_atomic, AtomicI8, i8, 1);
+atomic_int!(load_store_atomic, AtomicU8, u8, 1);
+atomic_int!(load_store_atomic, AtomicI16, i16, 2);
+atomic_int!(load_store_atomic, AtomicU16, u16, 2);
 
 #[cfg(not(target_pointer_width = "16"))]
-atomic_int!(int, load_store_atomic, AtomicI32, i32, 4);
+atomic_int!(load_store_atomic, AtomicI32, i32, 4);
 #[cfg(not(target_pointer_width = "16"))]
-atomic_int!(uint, load_store_atomic, AtomicU32, u32, 4);
+atomic_int!(load_store_atomic, AtomicU32, u32, 4);
 #[cfg(target_pointer_width = "16")]
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(int, load_store_critical_session, AtomicI32, i32, 4);
+atomic_int!(load_store_critical_session, AtomicI32, i32, 4);
 #[cfg(target_pointer_width = "16")]
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(uint, load_store_critical_session, AtomicU32, u32, 4);
+atomic_int!(load_store_critical_session, AtomicU32, u32, 4);
 
 #[cfg(not(any(target_pointer_width = "16", target_pointer_width = "32")))]
-atomic_int!(int, load_store_atomic, AtomicI64, i64, 8);
+atomic_int!(load_store_atomic, AtomicI64, i64, 8);
 #[cfg(not(any(target_pointer_width = "16", target_pointer_width = "32")))]
-atomic_int!(uint, load_store_atomic, AtomicU64, u64, 8);
+atomic_int!(load_store_atomic, AtomicU64, u64, 8);
 #[cfg(any(target_pointer_width = "16", target_pointer_width = "32"))]
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(int, load_store_critical_session, AtomicI64, i64, 8);
+atomic_int!(load_store_critical_session, AtomicI64, i64, 8);
 #[cfg(any(target_pointer_width = "16", target_pointer_width = "32"))]
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(uint, load_store_critical_session, AtomicU64, u64, 8);
+atomic_int!(load_store_critical_session, AtomicU64, u64, 8);
 
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(int, load_store_critical_session, AtomicI128, i128, 16);
+atomic_int!(load_store_critical_session, AtomicI128, i128, 16);
 #[cfg(any(test, feature = "fallback"))]
-atomic_int!(uint, load_store_critical_session, AtomicU128, u128, 16);
+atomic_int!(load_store_critical_session, AtomicU128, u128, 16);
 
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    test_atomic_bool_single_thread!();
     test_atomic_ptr_single_thread!();
     test_atomic_int_single_thread!(i8);
     test_atomic_int_single_thread!(u8);
@@ -819,6 +623,8 @@ mod tests {
     test_atomic_int_single_thread!(u32);
     test_atomic_int_single_thread!(i64);
     test_atomic_int_single_thread!(u64);
+    test_atomic_int_single_thread!(i128);
+    test_atomic_int_single_thread!(u128);
     test_atomic_int_single_thread!(isize);
     test_atomic_int_single_thread!(usize);
 }
